@@ -1,11 +1,12 @@
 import base64
 import time
 
-import dagster
 import requests.exceptions
-from dagster import check
 from databricks_api import DatabricksAPI
 
+import dagster
+import dagster._check as check
+
 from .types import (
     DATABRICKS_RUN_TERMINATED_STATES,
     DatabricksRunLifeCycleState,
@@ -13,7 +14,7 @@ from .types import (
 )
 
 # wait at most 24 hours by default for run execution
-_DEFAULT_RUN_MAX_WAIT_TIME_SEC = 24 * 60 * 60
+DEFAULT_RUN_MAX_WAIT_TIME_SEC = 24 * 60 * 60
 
 
 class DatabricksError(Exception):
@@ -32,7 +33,7 @@ class DatabricksClient:
         """Submit a run directly to the 'Runs Submit' API."""
         return self.client.jobs.submit_run(*args, **kwargs)["run_id"]  # pylint: disable=no-member
 
-    def read_file(self, dbfs_path, block_size=1024 ** 2):
+    def read_file(self, dbfs_path, block_size=1024**2):
         """Read a file from DBFS to a **byte string**."""
 
         if dbfs_path.startswith("dbfs://"):
@@ -49,7 +50,7 @@ class DatabricksClient:
             data += base64.b64decode(jdoc["data"])
         return data
 
-    def put_file(self, file_obj, dbfs_path, overwrite=False, block_size=1024 ** 2):
+    def put_file(self, file_obj, dbfs_path, overwrite=False, block_size=1024**2):
         """Upload an arbitrary large file to DBFS.
 
         This doesn't use the DBFS `Put` API because that endpoint is limited to 1MB.
@@ -69,13 +70,16 @@ class DatabricksClient:
 
         self.client.dbfs.close(handle=handle)  # pylint: disable=no-member
 
+    def get_run(self, databricks_run_id):
+        return self.client.jobs.get_run(databricks_run_id)  # pylint: disable=no-member
+
     def get_run_state(self, databricks_run_id):
         """Get the state of a run by Databricks run ID (_not_ dagster run ID).
 
         Return a `DatabricksRunState` object. Note that the `result_state`
         attribute may be `None` if the run hasn't yet terminated.
         """
-        run = self.client.jobs.get_run(databricks_run_id)  # pylint: disable=no-member
+        run = self.get_run(databricks_run_id)
         state = run["state"]
         result_state = state.get("result_state")
         if result_state:
@@ -111,7 +115,7 @@ class DatabricksJobRunner:
     """Submits jobs created using Dagster config to Databricks, and monitors their progress."""
 
     def __init__(
-        self, host, token, poll_interval_sec=10, max_wait_time_sec=_DEFAULT_RUN_MAX_WAIT_TIME_SEC
+        self, host, token, poll_interval_sec=5, max_wait_time_sec=DEFAULT_RUN_MAX_WAIT_TIME_SEC
     ):
         """Args:
         host (str): Databricks host, e.g. https://uksouth.azuredatabricks.net
@@ -169,8 +173,10 @@ class DatabricksJobRunner:
         # since they're imported by our scripts.
         # Add them if they're not already added by users in config.
         libraries = list(run_config.get("libraries", []))
-        python_libraries = {x["pypi"]["package"].split("==")[0] for x in libraries if "pypi" in x}
-        for library in ["dagster", "dagster_databricks", "dagster_pyspark"]:
+        python_libraries = {
+            x["pypi"]["package"].split("==")[0].replace("_", "-") for x in libraries if "pypi" in x
+        }
+        for library in ["dagster", "dagster-databricks", "dagster-pyspark"]:
             if library not in python_libraries:
                 libraries.append(
                     {"pypi": {"package": "{}=={}".format(library, dagster.__version__)}}
@@ -247,32 +253,43 @@ class DatabricksJobRunner:
         )
 
 
+def poll_run_state(
+    client,
+    log,
+    start_poll_time: float,
+    databricks_run_id: int,
+    max_wait_time_sec: float,
+):
+    run_state = client.get_run_state(databricks_run_id)
+    if run_state.has_terminated():
+        if run_state.is_successful():
+            log.info("Run %s completed successfully" % databricks_run_id)
+            return True
+        else:
+            error_message = "Run %s failed with result state: %s. Message: %s" % (
+                databricks_run_id,
+                run_state.result_state,
+                run_state.state_message,
+            )
+            log.error(error_message)
+            raise DatabricksError(error_message)
+    else:
+        log.debug("Run %s in state %s" % (databricks_run_id, run_state))
+    if time.time() - start_poll_time > max_wait_time_sec:
+        raise DatabricksError(
+            "Job run {} took more than {}s to complete; failing".format(
+                databricks_run_id, max_wait_time_sec
+            )
+        )
+    return False
+
+
 def wait_for_run_to_complete(client, log, databricks_run_id, poll_interval_sec, max_wait_time_sec):
     """Wait for a Databricks run to complete."""
     check.int_param(databricks_run_id, "databricks_run_id")
     log.info("Waiting for Databricks run %s to complete..." % databricks_run_id)
     start = time.time()
     while True:
-        log.debug("Waiting %.1f seconds..." % poll_interval_sec)
+        if poll_run_state(client, log, start, databricks_run_id, max_wait_time_sec):
+            return
         time.sleep(poll_interval_sec)
-        run_state = client.get_run_state(databricks_run_id)
-        if run_state.has_terminated():
-            if run_state.is_successful():
-                log.info("Run %s completed successfully" % databricks_run_id)
-                return
-            else:
-                error_message = "Run %s failed with result state: %s. Message: %s" % (
-                    databricks_run_id,
-                    run_state.result_state,
-                    run_state.state_message,
-                )
-                log.error(error_message)
-                raise DatabricksError(error_message)
-        else:
-            log.info("Run %s in state %s" % (databricks_run_id, run_state))
-        if time.time() - start > max_wait_time_sec:
-            raise DatabricksError(
-                "Job run {} took more than {}s to complete; failing".format(
-                    databricks_run_id, max_wait_time_sec
-                )
-            )
